# coding: utf-8
# © 2016-2021 Resurface Labs Inc.

import json
from typing import List, Optional

from usagelogger.base_logger import BaseLogger
from usagelogger.http_rules import HttpRules


class HttpLogger(BaseLogger):
    """Usage logger for HTTP/HTTPS protocol."""

    # Agent string identifying this logger.
    AGENT: str = "http_logger.py"

    def __init__(
        self,
        enabled: bool = True,
        queue: Optional[List[str]] = None,
        url: Optional[str] = None,
        skip_compression: bool = False,
        skip_submission: bool = False,
        rules: Optional[str] = None,
    ) -> None:

        super().__init__(
            self.AGENT,
            enabled=enabled,
            queue=queue,
            url=url,
            skip_compression=skip_compression,
            skip_submission=skip_submission,
        )

        # parse specified rules
        self._rules = HttpRules(rules)  # type: ignore

        # apply configuration rules
        self.skip_compression = self._rules.skip_compression
        self.skip_submission = self._rules.skip_submission
        if (
            self._enabled
            and url is not None
            and url.startswith("http:")
            and not self._rules.allow_http_url
        ):
            self._enableable = False
            self._enabled = False

    @property
    def rules(self) -> HttpRules:
        return self._rules

    def submit_if_passing(self, details: List[List[str]]) -> None:
        # apply active rules
        details = self._rules.apply(details)  # type: ignore
        if details is None:
            return

        # finalize message
        details.append(["host", self.host])

        # let's do this thing
        self.submit(json.dumps(details, separators=(",", ":")))



# coding: utf-8
# © 2016-2021 Resurface Labs Inc.

import time

from django.conf import settings

from usagelogger import HttpLogger, HttpMessage


def __read_settings__(key):
    try:
        return settings.USAGELOGGER[key]
    except Exception:
        return None


class HttpLoggerForDjango:
    def __init__(self, get_response):
        self.get_response = get_response
        self.logger = HttpLogger(
            url=__read_settings__("url"), rules=__read_settings__("rules")
        )

    def __call__(self, request):
        start_time = time.time()
        response = self.get_response(request)
        interval = str((time.time() - start_time) * 1000)

        HttpMessage.send(
            self.logger,
            request=request,
            response=response,
            interval=interval,
        )
        return response



# coding: utf-8
# © 2016-2021 Resurface Labs Inc.

from typing import Dict, Optional


class HttpResponseImpl(object):
    def __init__(
        self,
        status: Optional[int] = None,
        headers: Optional[Dict[str, str]] = None,
        body: Optional[str] = None,
    ) -> None:
        self.status = status
        self.headers = {} if headers is None else headers
        self.body = body



# coding: utf-8
# © 2016-2021 Resurface Labs Inc.

from typing import Dict, Optional


class HttpRequestImpl(object):
    def __init__(
        self,
        method: Optional[str] = None,
        url: Optional[str] = None,
        headers: Optional[Dict[str, str]] = None,
        params: Optional[Dict[str, str]] = None,
        body: Optional[str] = None,
    ) -> None:
        self.method = method
        self.url = url
        self.headers = {} if headers is None else headers
        self.params = {} if params is None else params
        self.body = body



# coding: utf-8
# © 2016-2021 Resurface Labs Inc.
import os
import socket
import threading
import zlib
from typing import Dict, List, Optional
from urllib.parse import urlsplit

import requests

import usagelogger  # just to read version
from usagelogger.usage_loggers import UsageLoggers


class BaseLogger:
    """Basic usage logger to embed or extend."""

    def __init__(
        self,
        agent: str,
        enabled: bool = True,
        queue: Optional[List[str]] = None,
        url: Optional[str] = None,
        skip_compression: bool = False,
        skip_submission: bool = False,
        conn=requests.Session(),
    ) -> None:

        self.agent = agent
        self.host = self.host_lookup()
        self.skip_compression = skip_compression
        self.skip_submission = skip_submission
        self.version = self.version_lookup()
        self.conn = conn

        # read provided options
        if url is None:
            url = UsageLoggers.url_by_default()

        # set options in priority order
        self._enabled = enabled
        self._queue = queue if isinstance(queue, list) else None
        if self._queue is not None:
            self._url = None
        elif url is not None and isinstance(url, str):
            try:
                if urlsplit(url).scheme in {"http", "https"}:
                    self._url_scheme: str = urlsplit(url).scheme
                    self._url = url
                else:
                    raise TypeError("incorrect URL scheme")
            except TypeError:
                self._enabled = False
                self._url = None
        else:
            self._enabled = False
            self._url = None

        # finalize internal properties
        self._enableable = self.queue is not None or self.url is not None
        self._submit_failures = 0
        self._submit_failures_lock = threading.Lock()
        self._submit_successes = 0
        self._submit_successes_lock = threading.Lock()

    def disable(self):
        self._enabled = False
        return self

    def enable(self):
        if self.enableable:
            self._enabled = True
        return self

    @property
    def enableable(self) -> bool:
        return self._enableable

    @property
    def enabled(self) -> bool:
        return self._enabled and UsageLoggers.is_enabled()

    @property
    def queue(self) -> Optional[List[str]]:
        return self._queue

    def submit(self, msg: str) -> None:
        """Submits JSON message to intended destination."""

        if msg is None or self.skip_submission is True or self.enabled is False:
            pass
        elif self._queue is not None:
            self._queue.append(msg)
            with self._submit_successes_lock:
                self._submit_successes += 1
        else:
            try:
                headers: Dict[str, str] = {
                    "Connection": "keep-alive",
                    "Content-Type": "application/json; charset=UTF-8",
                    "User-Agent": "Resurface/"
                    + usagelogger.__version__
                    + " ("
                    + self.agent
                    + ")",
                }

                if not self.skip_compression:
                    body = msg.encode("utf-8")
                else:
                    headers["Content-Encoding"] = "deflated"
                    body = zlib.compress(msg.encode("utf-8"))

                response = self.conn.post(self.url, data=body, headers=headers)
                if response.status_code == 204:
                    with self._submit_successes_lock:
                        self._submit_successes += 1
                else:
                    with self._submit_failures_lock:
                        self._submit_failures += 1

            # http errors
            except (requests.exceptions.RequestException, IOError, OSError):
                with self._submit_failures_lock:
                    self._submit_failures += 1
            # JSON errors
            except (OverflowError, TypeError, ValueError):
                with self._submit_failures_lock:
                    self._submit_failures += 1

    @property
    def submit_failures(self) -> int:
        return self._submit_failures

    @property
    def submit_successes(self) -> int:
        return self._submit_successes

    @property
    def url(self) -> str:
        return self._url  # type: ignore

    @staticmethod
    def host_lookup() -> str:
        dyno = os.getenv("DYNO")
        if dyno is not None:
            return dyno
        try:
            return socket.gethostname()
        except Exception:
            return "unknown"

    @staticmethod
    def version_lookup() -> str:
        return usagelogger.__version__



# coding: utf-8
# © 2016-2021 Resurface Labs Inc.
from typing import Optional, Pattern, Union


class HttpRule(object):
    def __init__(
        self,
        verb: str,
        scope: Optional[Pattern] = None,
        param1: Optional[Union[Pattern, str, int]] = None,
        param2: Optional[Union[Pattern, str]] = None,
    ) -> None:
        self._verb = verb
        self._scope = scope
        self._param1 = param1
        self._param2 = param2

    @property
    def verb(self):
        return self._verb

    @property
    def scope(self):
        return self._scope

    @property
    def param1(self):
        return self._param1

    @property
    def param2(self):
        return self._param2



import time
from typing import Optional

from aiohttp import web

from usagelogger import HttpLogger, HttpMessage
from usagelogger.http_request_impl import HttpRequestImpl
from usagelogger.http_response_impl import HttpResponseImpl


def HttpLoggerForAIOHTTP(url: Optional[str] = None, rules: Optional[str] = None):
    logger = HttpLogger(url=url, rules=rules)

    @web.middleware
    async def resurface_logger_middleware(request, handler):
        start_time = time.time()
        response = await handler(request)

        interval = str((time.time() - start_time) * 1000)
        data__: bytes = await request.read()

        HttpMessage.send(
            logger,
            request=HttpRequestImpl(
                url=str(request.url),
                headers=request.headers,
                params=request.query,
                method=request.method,
                body=data__.decode(),
            ),
            response=HttpResponseImpl(
                status=response.status,
                headers=response.headers,
                body=response.body.decode("utf8"),
            ),
            interval=interval,
        )

        return response

    return resurface_logger_middleware



# coding: utf-8
# © 2016-2021 Resurface Labs Inc.

import random
import re
from pathlib import Path
from typing import List, Optional, Pattern, Sized

from usagelogger.http_rule import HttpRule


class HttpRules(Sized):
    __DEBUG_RULES: str = r"allow_http_url" "\n" r"copy_session_field /.*/" "\n"

    __STANDARD_RULES: str = (
        "/request_header:cookie|response_header:set-cookie/ remove\n"
        r"/(request|response)_body|request_param/ replace "
        r"/[a-zA-Z0-9.!#$%&’*+\/=?^_`{|}~-]+@[a-zA-Z0-9-]+"
        r"(?:\.[a-zA-Z0-9-]+)/, /x@y.com/"
        "\n"
        r"/request_body|request_param|response_body/ replace "
        r"/[0-9\.\-\/]{9,}/, /xyxy/"
        "\n"
    )

    __STRICT_RULES: str = (
        r"/request_url/ replace /([^\?;]+).*/, !\\1!"
        "\n"
        r"/request_body|response_body|request_param:.*|"
        r"request_header:(?!user-agent).*|response_header:"
        r"(?!(content-length)|(content-type)).*/ remove"
        "\n"
    )

    __default_rules: str = __STRICT_RULES

    @classmethod
    def default_rules(cls) -> str:
        return cls.__default_rules

    @classmethod
    def set_default_rules(cls, rules: str) -> None:
        cls.__default_rules = re.sub(
            r"^\s*include default\s*$", "", rules, flags=re.MULTILINE
        )

    @classmethod
    def debug_rules(cls) -> str:
        return cls.__DEBUG_RULES

    @classmethod
    def standard_rules(cls) -> str:
        return cls.__STANDARD_RULES

    @classmethod
    def strict_rules(cls) -> str:
        return cls.__STRICT_RULES

    @classmethod
    def parse_rule(cls, rule: str) -> Optional[HttpRule]:
        """Parses rule from single line."""
        if rule is None or HttpRules.__REGEX_BLANK_OR_COMMENT.match(rule):
            return None
        m = HttpRules.__REGEX_ALLOW_HTTP_URL.match(rule)
        if m:
            return HttpRule("allow_http_url")
        m = HttpRules.__REGEX_COPY_SESSION_FIELD.match(rule)
        if m:
            return HttpRule(
                "copy_session_field", None, HttpRules.parse_regex(rule, m.group(1))
            )
        m = HttpRules.__REGEX_REMOVE.match(rule)
        if m:
            return HttpRule("remove", HttpRules.parse_regex(rule, m.group(1)))
        m = HttpRules.__REGEX_REMOVE_IF.match(rule)
        if m:
            return HttpRule(
                "remove_if",
                HttpRules.parse_regex(rule, m.group(1)),
                HttpRules.parse_regex(rule, m.group(2)),
            )
        m = HttpRules.__REGEX_REMOVE_IF_FOUND.match(rule)
        if m:
            return HttpRule(
                "remove_if_found",
                HttpRules.parse_regex(rule, m.group(1)),
                HttpRules.parse_regex_find(rule, m.group(2)),
            )
        m = HttpRules.__REGEX_REMOVE_UNLESS.match(rule)
        if m:
            return HttpRule(
                "remove_unless",
                HttpRules.parse_regex(rule, m.group(1)),
                HttpRules.parse_regex(rule, m.group(2)),
            )
        m = HttpRules.__REGEX_REMOVE_UNLESS_FOUND.match(rule)
        if m:
            return HttpRule(
                "remove_unless_found",
                HttpRules.parse_regex(rule, m.group(1)),
                HttpRules.parse_regex_find(rule, m.group(2)),
            )
        m = HttpRules.__REGEX_REPLACE.match(rule)
        if m:
            return HttpRule(
                "replace",
                HttpRules.parse_regex(rule, m.group(1)),
                HttpRules.parse_regex_find(rule, m.group(2)),
                HttpRules.parse_string(rule, m.group(3)),
            )
        m = HttpRules.__REGEX_SAMPLE.match(rule)
        if m:
            m1 = int(m.group(1))
            if m1 < 1 or m1 > 99:
                raise SyntaxError("Invalid sample percent: " + m.group(1))
            return HttpRule("sample", None, m1)
        m = HttpRules.__REGEX_SKIP_COMPRESSION.match(rule)
        if m:
            return HttpRule("skip_compression")
        m = HttpRules.__REGEX_SKIP_SUBMISSION.match(rule)
        if m:
            return HttpRule("skip_submission")
        m = HttpRules.__REGEX_STOP.match(rule)
        if m:
            return HttpRule("stop", HttpRules.parse_regex(rule, m.group(1)))
        m = HttpRules.__REGEX_STOP_IF.match(rule)
        if m:
            return HttpRule(
                "stop_if",
                HttpRules.parse_regex(rule, m.group(1)),
                HttpRules.parse_regex(rule, m.group(2)),
            )
        m = HttpRules.__REGEX_STOP_IF_FOUND.match(rule)
        if m:
            return HttpRule(
                "stop_if_found",
                HttpRules.parse_regex(rule, m.group(1)),
                HttpRules.parse_regex_find(rule, m.group(2)),
            )
        m = HttpRules.__REGEX_STOP_UNLESS.match(rule)
        if m:
            return HttpRule(
                "stop_unless",
                HttpRules.parse_regex(rule, m.group(1)),
                HttpRules.parse_regex(rule, m.group(2)),
            )
        m = HttpRules.__REGEX_STOP_UNLESS_FOUND.match(rule)
        if m:
            return HttpRule(
                "stop_unless_found",
                HttpRules.parse_regex(rule, m.group(1)),
                HttpRules.parse_regex_find(rule, m.group(2)),
            )
        raise SyntaxError("Invalid rule: " + rule)

    @classmethod
    def parse_regex(cls, rule: str, regex: str) -> Pattern:
        """Parses regex for matching."""
        s: str = HttpRules.parse_string(rule, regex)
        if not s.startswith("^"):
            s = "^" + s
        if not s.endswith("$"):
            s = s + "$"
        try:
            return re.compile(s)
        except Exception:
            raise SyntaxError(f"Invalid regex ({regex}) in rule: {rule}")

    @classmethod
    def parse_regex_find(cls, rule: str, regex: str) -> Pattern:
        """Parses regex for finding."""
        try:
            return re.compile(HttpRules.parse_string(rule, regex))
        except Exception:
            raise SyntaxError(f"Invalid regex ({regex}) in rule: {rule}")

    @classmethod
    def parse_string(cls, rule: str, expr: str) -> str:
        """Parses delimited string expression."""
        for sep in ["~", "!", "%", "|", "/"]:
            p: Pattern = re.compile(r"^[{s}](.*)[{s}]$".format(s=sep))
            m = p.match(expr)
            if m:
                m1: str = m.group(1)
                m1p: Pattern = re.compile(r"^[{s}].*|.*[^\\\\][{s}].*".format(s=sep))
                if m1p.match(m1):
                    raise SyntaxError(f"Unescaped separator ({sep}) in rule: {rule}")
                return sep.join(m1.split("\\" + sep))
        raise SyntaxError(f"Invalid expression ({expr}) in rule: {rule}")

    def __init__(self, rules: str) -> None:
        if rules is None:
            rules = HttpRules.default_rules()

        # load rules from external files
        if rules.startswith("file://"):
            rfile = rules[7:]
            try:
                rules = Path(rfile).read_text()
            except Exception:
                raise FileNotFoundError(f"Failed to load rules: {rfile}")

        # force default rules if necessary
        rules = re.sub(
            r"^\s*include default\s*$",
            str(HttpRules.default_rules()),
            rules,
            flags=re.MULTILINE,
        )
        if len(rules.strip()) == 0:
            rules = HttpRules.default_rules()

        # expand rule includes
        rules = re.sub(r"(?m)^\s*include debug\s*$", HttpRules.debug_rules(), rules)
        rules = re.sub(
            r"(?m)^\s*include standard\s*$", HttpRules.standard_rules(), rules
        )
        rules = re.sub(r"(?m)^\s*include strict\s*$", HttpRules.strict_rules(), rules)
        self._text = rules

        # parse all rules
        prs: List = []
        for rule in rules.split("\n"):
            parsed = HttpRules.parse_rule(rule)
            if parsed is not None:
                prs.append(parsed)
        self._length = len(prs)

        # break out rules by verb
        self._allow_http_url: bool = (
            len([r for r in prs if "allow_http_url" == r.verb]) > 0
        )
        self._copy_session_field: List[HttpRule] = [
            r for r in prs if "copy_session_field" == r.verb
        ]
        self._remove: List[HttpRule] = [r for r in prs if "remove" == r.verb]
        self._remove_if: List[HttpRule] = [r for r in prs if "remove_if" == r.verb]
        self._remove_if_found: List[HttpRule] = [
            r for r in prs if "remove_if_found" == r.verb
        ]
        self._remove_unless: List[HttpRule] = [
            r for r in prs if "remove_unless" == r.verb
        ]
        self._remove_unless_found: List[HttpRule] = [
            r for r in prs if "remove_unless_found" == r.verb
        ]
        self._replace: List[HttpRule] = [r for r in prs if "replace" == r.verb]
        self._sample: List[HttpRule] = [r for r in prs if "sample" == r.verb]
        self._skip_compression: bool = (
            len([r for r in prs if r.verb == "skip_compression"]) > 0
        )
        self._skip_submission: bool = (
            len([r for r in prs if r.verb == "skip_submission"]) > 0
        )
        self._stop: List[HttpRule] = [r for r in prs if "stop" == r.verb]
        self._stop_if: List[HttpRule] = [r for r in prs if "stop_if" == r.verb]
        self._stop_if_found: List[HttpRule] = [
            r for r in prs if "stop_if_found" == r.verb
        ]
        self._stop_unless: List[HttpRule] = [r for r in prs if "stop_unless" == r.verb]
        self._stop_unless_found: List[HttpRule] = [
            r for r in prs if "stop_unless_found" == r.verb
        ]

        # validate rules
        if len(self._sample) > 1:
            raise SyntaxError("Multiple sample rules")

    def __len__(self) -> int:
        return self._length

    @property
    def allow_http_url(self) -> bool:
        return self._allow_http_url

    @property
    def copy_session_field(self) -> List[HttpRule]:
        return self._copy_session_field

    @property
    def remove(self) -> List[HttpRule]:
        return self._remove

    @property
    def remove_if(self) -> List[HttpRule]:
        return self._remove_if

    @property
    def remove_if_found(self) -> List[HttpRule]:
        return self._remove_if_found

    @property
    def remove_unless(self) -> List[HttpRule]:
        return self._remove_unless

    @property
    def remove_unless_found(self) -> List[HttpRule]:
        return self._remove_unless_found

    @property
    def replace(self) -> List[HttpRule]:
        return self._replace

    @property
    def sample(self) -> List[HttpRule]:
        return self._sample

    @property
    def skip_compression(self) -> bool:
        return self._skip_compression

    @property
    def skip_submission(self) -> bool:
        return self._skip_submission

    @property
    def stop(self) -> List[HttpRule]:
        return self._stop

    @property
    def stop_if(self) -> List[HttpRule]:
        return self._stop_if

    @property
    def stop_if_found(self) -> List[HttpRule]:
        return self._stop_if_found

    @property
    def stop_unless(self) -> List[HttpRule]:
        return self._stop_unless

    @property
    def stop_unless_found(self) -> List[HttpRule]:
        return self._stop_unless_found

    @property
    def text(self) -> str:
        return self._text

    def apply(self, details: List[List[str]]) -> Optional[List[List[str]]]:
        # stop rules come first
        for r in self._stop:
            for d in details:
                if r.scope.match(d[0]):
                    return None
        for r in self._stop_if_found:
            for d in details:
                if r.scope.match(d[0]) and r.param1.search(d[1]):
                    return None
        for r in self._stop_if:
            for d in details:
                if r.scope.match(d[0]) and r.param1.match(d[1]):
                    return None
        passed = 0
        for r in self._stop_unless_found:
            for d in details:
                if r.scope.match(d[0]) and r.param1.search(d[1]):
                    passed += 1
        if passed != len(self._stop_unless_found):
            return None
        passed = 0
        for r in self._stop_unless:
            for d in details:
                if r.scope.match(d[0]) and r.param1.match(d[1]):
                    passed += 1
        if passed != len(self._stop_unless):
            return None

        # do sampling if configured
        if len(self._sample) == 1 and random.randrange(100) >= int(
            self._sample[0].param1
        ):
            return None

        # winnow sensitive details based on remove rules if configured
        for r in self._remove:
            for d in details:
                if r.scope.match(d[0]):
                    d[1] = ""
        for r in self._remove_unless_found:
            for d in details:
                if r.scope.match(d[0]) and not r.param1.search(d[1]):
                    d[1] = ""
        for r in self._remove_if_found:
            for d in details:
                if r.scope.match(d[0]) and r.param1.search(d[1]):
                    d[1] = ""
        for r in self._remove_unless:
            for d in details:
                if r.scope.match(d[0]) and not r.param1.match(d[1]):
                    d[1] = ""
        for r in self._remove_if:
            for d in details:
                if r.scope.match(d[0]) and r.param1.match(d[1]):
                    d[1] = ""

        # remove any details with empty values
        details = [x for x in details if x[1] != ""]
        if len(details) == 0:
            return None

        # mask sensitive details based on replace rules if configured
        for r in self._replace:
            for d in details:
                if r.scope.match(d[0]):
                    d[1] = re.sub(r.param1, r.param2, d[1])

        # remove any details with empty values
        details = [x for x in details if x[1] != ""]
        return None if len(details) == 0 else details

    __REGEX_ALLOW_HTTP_URL: Pattern = re.compile(r"^\s*allow_http_url\s*(#.*)?$")
    __REGEX_BLANK_OR_COMMENT: Pattern = re.compile(r"^\s*([#].*)*$")
    __REGEX_COPY_SESSION_FIELD: Pattern = re.compile(
        r"^\s*copy_session_field\s+([~!%|/].+[~!%|/])\s*(#.*)?$"
    )
    __REGEX_REMOVE: Pattern = re.compile(r"^\s*([~!%|/].+[~!%|/])\s*remove\s*(#.*)?$")
    __REGEX_REMOVE_IF: Pattern = re.compile(
        r"^\s*([~!%|/].+[~!%|/])\s*remove_if\s+([~!%|/].+[~!%|/])\s*(#.*)?$"
    )
    __REGEX_REMOVE_IF_FOUND: Pattern = re.compile(
        r"^\s*([~!%|/].+[~!%|/])\s*" r"remove_if_found\s+([~!%|/].+[~!%|/])\s*(#.*)?$"
    )
    __REGEX_REMOVE_UNLESS: Pattern = re.compile(
        r"^\s*([~!%|/].+[~!%|/])\s*" r"remove_unless\s+([~!%|/].+[~!%|/])\s*(#.*)?$"
    )
    __REGEX_REMOVE_UNLESS_FOUND: Pattern = re.compile(
        r"^\s*([~!%|/].+[~!%|/])\s*"
        r"remove_unless_found\s+([~!%|/].+[~!%|/])\s*(#.*)?$"
    )
    __REGEX_REPLACE: Pattern = re.compile(
        r"^\s*([~!%|/].+[~!%|/])\s*"
        r"replace[\s]+([~!%|/].+[~!%|/]),[\s]+([~!%|/].*[~!%|/])\s*(#.*)?$"
    )
    __REGEX_SAMPLE: Pattern = re.compile(r"^\s*sample\s+(\d+)\s*(#.*)?$")
    __REGEX_SKIP_COMPRESSION: Pattern = re.compile(r"^\s*skip_compression\s*(#.*)?$")
    __REGEX_SKIP_SUBMISSION: Pattern = re.compile(r"^\s*skip_submission\s*(#.*)?$")
    __REGEX_STOP: Pattern = re.compile(r"^\s*([~!%|/].+[~!%|/])\s*stop\s*(#.*)?$")
    __REGEX_STOP_IF: Pattern = re.compile(
        r"^\s*([~!%|/].+[~!%|/])\s*stop_if\s+([~!%|/].+[~!%|/])\s*(#.*)?$"
    )
    __REGEX_STOP_IF_FOUND: Pattern = re.compile(
        r"^\s*([~!%|/].+[~!%|/])\s*" r"stop_if_found\s+([~!%|/].+[~!%|/])\s*(#.*)?$"
    )
    __REGEX_STOP_UNLESS: Pattern = re.compile(
        r"^\s*([~!%|/].+[~!%|/])\s*stop_unless\s+([~!%|/].+[~!%|/])\s*(#.*)?$"
    )
    __REGEX_STOP_UNLESS_FOUND: Pattern = re.compile(
        r"^\s*([~!%|/].+[~!%|/])\s*" r"stop_unless_found\s+([~!%|/].+[~!%|/])\s*(#.*)?$"
    )



# coding: utf-8
# © 2016-2021 Resurface Labs Inc.

from os import getenv
from typing import Optional


class UsageLoggers(object):
    __BRICKED: bool = getenv("USAGE_LOGGERS_DISABLE") == "true"

    __disabled: bool = __BRICKED

    @classmethod
    def disable(cls) -> None:
        cls.__disabled = True

    @classmethod
    def enable(cls) -> None:
        if cls.__BRICKED is False:
            cls.__disabled = False

    @classmethod
    def is_enabled(cls) -> bool:
        return not cls.__disabled

    @staticmethod
    def url_by_default() -> Optional[str]:
        return getenv("USAGE_LOGGERS_URL")



from .base_logger import BaseLogger  # noqa
from .http_logger import HttpLogger  # noqa
from .http_message import HttpMessage  # noqa
from .http_request_impl import HttpRequestImpl  # noqa
from .http_response_impl import HttpResponseImpl  # noqa
from .http_rules import HttpRules  # noqa
from .usage_loggers import UsageLoggers  # noqa

__version__ = "2.2.3"

__all___ = [
    "UsageLoggers",
    "HttpRequestImpl",
    "HttpResponseImpl",
    "HttpRules",
    "BaseLogger",
    "HttpLogger",
    "HttpMessage",
]



"""The following code is adapted from git@github.com:jmcarp/requests-middleware.git under the MIT license.

Copyright 2014 Joshua Carp

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE."""


from requests import Response
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.poolmanager import PoolManager
from requests.packages.urllib3.response import HTTPResponse


class MiddlewareHTTPAdapter(HTTPAdapter):
    """An HTTPAdapter onto which :class:`BaseMiddleware <BaseMiddleware>`
    can be registered. Middleware methods are called in the order of
    registration. Note: contrib that expose actions called during adapter
    initialization must be passed to `__init__` rather than `register`, else
    those actions will not take effect.
    :param list middlewares: List of :class:`BaseMiddleware <BaseMiddleware>`
        objects
    """

    def __init__(self, middlewares=None, *args, **kwargs):
        self.middlewares = middlewares or []
        super(MiddlewareHTTPAdapter, self).__init__(*args, **kwargs)

    def register(self, middleware):
        """Add a middleware to the middleware stack.
        :param BaseMiddleware middleware: The middleware object
        """
        self.middlewares.append(middleware)

    def init_poolmanager(self, connections, maxsize, block=False):
        """Assemble keyword arguments to be passed to `PoolManager`.
        Middlewares are called in reverse order, so if multiple middlewares
        define conflicting arguments, the higher-priority middleware will take
        precedence. Note: Arguments are passed directly to `PoolManager` and
        not to the superclass `init_poolmanager` because the superclass method
        does not currently accept **kwargs.
        """
        kwargs = {}
        for middleware in self.middlewares[::-1]:
            value = middleware.before_init_poolmanager(connections, maxsize, block)
            kwargs.update(value or {})

        self._pool_connections = connections
        self._pool_maxsize = maxsize
        self._pool_block = block

        self.poolmanager = PoolManager(
            num_pools=connections, maxsize=maxsize, block=block, **kwargs
        )

    def send(self, request, *args, **kwargs):
        """Send the request. If any middleware in the stack returns a `Response`
        or `HTTPResponse` value from its `before_send` method, short-circuit;
        else delegate to `HTTPAdapter::send`.
        :param request: The :class:`PreparedRequest <PreparedRequest>`
            being sent.
        :returns: The :class:`Response <Response>` object.
        """
        for middleware in self.middlewares:
            value = middleware.before_send(request, **kwargs)
            if isinstance(value, Response):
                return value
            if isinstance(value, HTTPResponse):
                return self.build_response(request, value)
            if value:
                raise ValueError(
                    'Middleware "before_send" methods must return '
                    "`Response`, `HTTPResponse`, or `None`"
                )
        return super(MiddlewareHTTPAdapter, self).send(request, *args, **kwargs)

    def build_response(self, req, resp):
        """Build the response. Call `HTTPAdapter::build_response`, then pass
        the response object to the `after_build_response` method of each
        middleware in the stack, in reverse order.
        :param req: The :class:`PreparedRequest <PreparedRequest>` used to
            generate the response.
        :param resp: The urllib3 response object.
        :returns: The :class:`Response <Response>` object.
        """
        for middleware in reversed(self.middlewares):
            req, resp = middleware.before_build_response(req, resp)
        response = super(MiddlewareHTTPAdapter, self).build_response(req, resp)
        for middleware in reversed(self.middlewares):
            response = middleware.after_build_response(req, resp, response)
        return response



# coding: utf-8
# © 2016-2021 Resurface Labs Inc.


import time
from io import BytesIO
from typing import Dict, Iterable, List, Optional, Tuple
from urllib import parse

from werkzeug.wrappers import Request
from werkzeug.wsgi import ClosingIterator

from usagelogger import HttpLogger, HttpMessage, HttpRequestImpl, HttpResponseImpl


class HttpLoggerForFlask:
    def __init__(self, app, url: Optional[str] = None, rules: Optional[str] = None):
        self.app = app
        self.logger = HttpLogger(url=url, rules=rules)
        self.start_time: float = 0.0
        self.interval: float = 0.0
        self.response: List[bytes] = []
        self.response_headers: Iterable[Tuple[str, str]] = []
        self.status: Optional[int] = None

    def start_response(
        self, status: str, response_headers: Iterable[Tuple[str, str]]
    ) -> None:
        self.start_time = time.time()
        self.status = int(status.split(" ")[0])
        self.response_headers = response_headers

    def finish_response(self, response: ClosingIterator) -> List[bytes]:
        self.interval = 1000.0 * (time.time() - self.start_time)
        stored_response_chunks = []
        for line in response:
            stored_response_chunks.append(line)
        self.response = stored_response_chunks
        return stored_response_chunks

    def request_body(self, environ):
        content_length = environ.get("CONTENT_LENGTH")
        body = ""
        if content_length:
            if content_length == "-1":
                # This is a special case, where the content length is basically undetermined
                body = environ["wsgi.input"].read(-1)
                content_length = len(body)
            else:
                content_length = int(content_length)
                body = environ["wsgi.input"].read(content_length)
                environ["wsgi.input"] = BytesIO(bytes(body))

        return body.decode("utf-8")

    def __call__(self, environ, start_response) -> ClosingIterator:

        body__ = self.request_body(environ)

        def _start_response(status, response_headers, *args):
            self.start_response(status, response_headers)
            return start_response(status, response_headers, *args)

        response_chunks = self.finish_response(self.app(environ, _start_response))
        request = Request(environ)

        parased_raw_params: Dict[str, List[str]] = parse.parse_qs(
            parse.urlparse(request.url).query
        )

        params: Dict[str, str] = {}

        # Type correction
        for k, v in parased_raw_params.items():
            params[k] = v[0]

        HttpMessage.send(
            self.logger,
            request=HttpRequestImpl(
                method=request.method,
                url=str(request.url),
                headers=dict(request.headers),
                params=params,
                body=body__,
            ),
            response=HttpResponseImpl(
                status=self.status,
                body=str(self.response[0].decode()) if self.response else None,
                headers=dict(self.response_headers),
            ),
            interval=str(self.interval),
        )

        return ClosingIterator(response_chunks)



from setuptools import find_packages, setup


def read_file(name):
    with open(name) as fd:
        return fd.read()


setup(
    name="usagelogger",
    version="2.2.3",
    description="Logging usage of Python-based services, with user privacy by design.",
    long_description=read_file("DESCRIPTIONS.md"),
    long_description_content_type="text/markdown",
    url="https://resurface.io",
    author="Resurface Labs",
    author_email="admin@resurface.io",
    license="Apache License 2.0",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: System Administrators",
        "Topic :: Internet :: Log Analysis",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    keywords="logging resurface",
    packages=find_packages(exclude=["tests"]),
    python_requires=">=3.7, <4",
    install_requires=read_file("requirements.txt").splitlines(),
    include_package_data=True,
    tests_require=["pytest"],
    project_urls={
        "Bug Reports": "https://github.com/resurfaceio/logger-python/issues",
        "Source": "https://github.com/resurfaceio/logger-python",
    },
)



# coding: utf-8
# © 2016-2021 Resurface Labs Inc.

from time import time
from typing import List, Optional
from urllib import parse

from usagelogger import HttpLogger


class HttpMessage(object):
    @classmethod
    def send(
        cls,
        logger: HttpLogger,
        request,
        response,
        response_body: Optional[str] = None,
        request_body: Optional[str] = None,
        now=None,
        interval=None,
    ) -> None:  # TODO: missing type hints

        if not logger.enabled:
            return

        # copy details from request & resonse
        message: List[List[str]] = cls.build(
            request, response, response_body, request_body
        )

        # TODO: copy details from active session

        # add timing details
        message.append(
            ["now", str(now) if now is not None else str(round(time() * 1000))]
        )
        if interval is not None:
            message.append(["interval", interval])

        logger.submit_if_passing(message)

    @classmethod
    def build(
        cls,
        request,
        response,
        response_body: Optional[str] = None,
        request_body: Optional[str] = None,
    ) -> List[List[str]]:

        message: List[List[str]] = []

        if request.__class__.__name__ == "WSGIRequest":
            message = []
            if request.method:
                message.append(["request_method", request.method])
            url = request.build_absolute_uri()
            if url:
                message.append(["request_url", url])
            if response.status_code:
                message.append(["response_code", str(response.status_code)])
            for k, v in request.headers.items():
                message.append([f"request_header:{k}".lower(), v])
            message.append(["request_body", request.body.decode()])
            if request.method == "GET":
                for k, v in request.GET.items():
                    message.append([f"request_param:{k}".lower(), v])
            elif request.method == "POST":
                for k, v in request.POST.items():
                    message.append([f"request_param:{k}".lower(), v])
            for k, v in response.items():
                message.append([f"response_header:{k}".lower(), v])
            message.append(["response_body", response.content.decode("utf8")])

        elif request.__class__.__name__ == "HttpRequestImpl":
            message = []
            if request.method:
                message.append(["request_method", request.method])
            if request.url:
                message.append(["request_url", request.url])
            if response.status:
                message.append(["response_code", str(response.status)])
            for k, v in request.headers.items():
                message.append([f"request_header:{k}".lower(), v])
            for k, v in request.params.items():
                message.append([f"request_param:{k}".lower(), v])
            for k, v in response.headers.items():
                message.append([f"response_header:{k}".lower(), v])
            final_request_body = (
                request_body if (request_body is not None) else request.body
            )
            if final_request_body:
                message.append(["request_body", final_request_body])
            final_response_body = (
                response_body if (response_body is not None) else response.body
            )
            if final_response_body:
                message.append(["response_body", final_response_body])

        elif request.__class__.__name__ == "PreparedRequest":
            message = []
            if request.method:
                message.append(["request_method", request.method])

            url = str(request.url)
            if url:
                message.append(["request_url", url])
            if response.status_code:
                message.append(["response_code", str(response.status_code)])
            for k, v in request.headers.items():
                message.append([f"request_header:{k}".lower(), v])
            if request.method == "GET":
                parsed_url = parse.parse_qs(parse.urlparse(url).query)
                for k, v in parsed_url.items():
                    message.append([f"request_param:{k}".lower(), v[0]])
            if request.body:
                message.append(["request_body", request.body])
            elif request.method == "POST":

                for data in request.body.split("&"):
                    k, v = data.split("=")
                    message.append([f"request_param:{k}".lower(), v])

            for k, v in response.headers.items():
                message.append([f"response_header:{k}".lower(), v])
            message.append(["response_body", response.content.decode("utf8")])

        return message



import time
from collections import OrderedDict
from typing import Optional

from requests.cookies import cookiejar_from_dict
from requests.hooks import default_hooks
from requests.models import DEFAULT_REDIRECT_LIMIT
from requests.sessions import Session as RequestsSession
from requests.utils import default_headers

from usagelogger import HttpLogger, HttpMessage

from ._adapter import MiddlewareHTTPAdapter


class Session(RequestsSession):
    def __init__(self, url: Optional[str] = None, rules: Optional[str] = None) -> None:
        self.headers = default_headers()

        self.auth = None

        self.proxies = {}

        self.hooks = default_hooks()

        self.params = {}

        self.stream = False

        self.verify = True

        self.cert = None

        self.max_redirects = DEFAULT_REDIRECT_LIMIT

        self.trust_env = True

        self.cookies = cookiejar_from_dict({})

        self.adapters = OrderedDict()

        middlewares = [ResurfaceHTTPAdapter(url=url, rules=rules)]

        adapter = MiddlewareHTTPAdapter(middlewares)

        self.mount("https://", adapter)
        self.mount("http://", adapter)


class ResurfaceHTTPAdapter:
    def __init__(
        self, url: Optional[str] = None, rules: Optional[str] = None, *args, **kwargs
    ):
        self.logger = HttpLogger(url=url, rules=rules)
        self.start_time = 0

    def before_init_poolmanager(self, connections, maxsize, block=False):
        """Called before `HTTPAdapter::init_poolmanager`. Optionally return a
        dictionary of keyword arguments to `PoolManager`.
        :returns: `dict` of keyword arguments or `None`
        """

    def before_send(self, request, *args, **kwargs):
        """Called before `HTTPAdapter::send`. If a truthy value is returned,
        :class:`MiddlewareHTTPAdapter <MiddlewareHTTPAdapter>` will short-
        circuit the remaining middlewares and `HTTPAdapter::send`, using the
        returned value instead.
        :param request: The `PreparedRequest` used to generate the response.
        :returns: The `Response` object or `None`.
        """

    def before_build_response(self, req, resp):
        self.start_time = time.time()
        return req, resp

    def after_build_response(self, req, resp, response):

        interval = str((time.time() - self.start_time) * 1000)
        HttpMessage.send(
            self.logger,
            request=req,
            response=response,
            interval=interval,
        )
        return response
